-----------------------------------------------------------------------------------------------
-- 'SAS' v1.5a (LAS Switcher and AutoSwitcher) [19/08/2014]
-----------------------------------------------------------------------------------------------

require "Window"
require "AbilityBook"
require "GameLib"
require "MatchingGame"

-----------------------------------------------------------------------------------------------
-- Module Definition
-----------------------------------------------------------------------------------------------
local SAS = {} 
local	L = Apollo.GetPackage("Gemini:Locale-1.0").tPackage:GetLocale("SAS", true)

-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
local 	SAS_VER = "1.5a"

-----------------------------------------------------------------------------------------------
-- new
-----------------------------------------------------------------------------------------------
function SAS:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 
    	
	o.tSasConfig = {}
	o.tSasConfig[1] = { pvp = false, pve = false, stance = 0, er = false }
	o.tSasConfig[2] = { pvp = false, pve = false, stance = 0, er = false }
	o.tSasConfig[3] = { pvp = false, pve = false, stance = 0, er = false }
	o.tSasConfig[4] = { pvp = false, pve = false, stance = 0, er = false }
	o.tSasConfig.link = false
	o.tSasConfig.er_start = 40
	o.tSasConfig.er_prev = 100
	o.tSasConfig.er_useprev = false
			
    return o
end

-----------------------------------------------------------------------------------------------
-- Init
-----------------------------------------------------------------------------------------------
function SAS:Init()
   	Apollo.RegisterAddon(self, false)
end
 
-----------------------------------------------------------------------------------------------
-- OnLoad
-----------------------------------------------------------------------------------------------
function SAS:OnLoad()
	
	self.Xml = XmlDoc.CreateFromFile("SAS.xml")
	self.wndSas = Apollo.LoadForm(self.Xml, "SASForm", nil, self)
	
    self.wndSas:Show(true)
   
	Apollo.RegisterEventHandler("SpecChanged", 			"OnSpecChanged", self)
	Apollo.RegisterEventHandler("StanceChanged", 		"OnStanceChanged", self)
	Apollo.RegisterEventHandler("UnitEnteredCombat", 	"OnEnteredCombat", self)
	Apollo.RegisterEventHandler("ChangeWorld",			"OnChangeWorld", self)
	 
	Apollo.RegisterTimerHandler("GetSpec_Timer", "OnGetSpe_Timer", self)
	Apollo.CreateTimer("GetSpec_Timer", 2 , false)
	Apollo.RegisterTimerHandler("NewSpe_Timer", "OnNewSpe_Timer", self)
	Apollo.RegisterTimerHandler("PrevSpe_Timer", "OnPrevSpe_Timer", self)
	
	Apollo.RegisterSlashCommand("sas", "OnSlashCommand", self)
	
	self.bHasStance = nil
	self.bInCombat = false
end

-----------------------------------------------------------------------------------------------
-- OnSlashCommand 
-----------------------------------------------------------------------------------------------
function SAS:OnSlashCommand(sCommand, sLas)
	
    local nSpe = AbilityBook.GetNumUnlockedSpecs()
    local nLas = tonumber(sLas) 
    
	if nLas == nil or nLas > nSpe then 
		 self:OnSetAction(nil, nil, nil)
	else
		self.nNewSpe = nLas
		AbilityBook.SetCurrentSpec(self.nNewSpe)
    end
end


-----------------------------------------------------------------------------------------------
-- OnStanceChanged
-----------------------------------------------------------------------------------------------
function SAS:OnStanceChanged() 

	self.nStance = GameLib.GetCurrentClassInnateAbilityIndex()
	self:DoSwitchSPE("Stance")
		
end

-----------------------------------------------------------------------------------------------
-- SetStanceBtn
-----------------------------------------------------------------------------------------------
function SAS:SetStanceBtn() 
    
    self.wndFrame = Apollo.LoadForm(self.Xml, "Frame_Wnd", self.wndSas:FindChild("OptionWnd_Frames"), self) 
    self.wndFrame:SetName("Frame2_Wnd")
    self.wndTitle_Stance = Apollo.LoadForm(self.Xml, "TitleWnd", self.wndSas:FindChild("Frame2_Wnd"), self)  
    self.wndTitle_Stance:SetText(L["SAS_SWITCHBYSTANCE"])
	
	local tStances = GameLib.GetClassInnateAbilitySpells()
	self.nStanceCount = tStances.nSpellCount
	self.arStances = {}
			 		
	for _ = 1, self.nStanceCount do
		self.arStances[_] = 
		{
		   	splPassive = tStances.tSpells[2 * _],
			wndStance = Apollo.LoadForm(self.Xml, "OptionForm", self.wndSas:FindChild("Frame2_Wnd"), self),
		}
		
		self.arStances[_].wndStance:FindChild("OptionBtn"):SetData(_)
		
		local sCut = string.sub(self.arStances[_].splPassive:GetName(), tostring(string.find(self.arStances[_].splPassive:GetName(), ":")), -1)
        local sStanceName = string.gsub(sCut,":" , "")
      	self.arStances[_].wndStance:SetText(sStanceName)
	end
		
	self.wndLink_Stance = Apollo.LoadForm(self.Xml, "LinkForm", self.wndSas:FindChild("Frame2_Wnd"), self)  
    self.wndLink_Stance:FindChild("LinkBtn"):SetData(self.nStanceCount + 1)
   	self.wndLink_Stance:SetText(L["SAS_LINK"])
	self.wndSas:FindChild("Frame2_Wnd"):ArrangeChildrenVert(1)
end

-----------------------------------------------------------------------------------------------
-- SetZoneBtn
-----------------------------------------------------------------------------------------------
function SAS:SetZoneBtn() 
	
    self.wndFrame = Apollo.LoadForm(self.Xml, "Frame_Wnd", self.wndSas:FindChild("OptionWnd_Frames"), self) 
    self.wndFrame:SetName("Frame1_Wnd")

    self.wndTitle_Zone = Apollo.LoadForm(self.Xml, "TitleWnd", self.wndSas:FindChild("Frame1_Wnd"), self)  
	self.wndTitle_Zone:SetText(L["SAS_SWITCHBYZONE"])
			
	local tZoneName = {}
		  tZoneName[1] = L["SAS_PVPZONE"]		
		  tZoneName[2] = L["SAS_PVEZONE"]		
		
	self.arZones = {}
			
	for _ = 1, 2 do
	    self.arZones[_] = 
		{
		   	sName = tZoneName[_],
			wndZone = Apollo.LoadForm(self.Xml, "OptionForm", self.wndSas:FindChild("Frame1_Wnd"), self)

		}
		
		self.arZones[_].wndZone:FindChild("OptionBtn"):SetData(_)
		self.arZones[_].wndZone:SetText(self.arZones[_].sName)
   	end	

	self.wndSas:FindChild("Frame1_Wnd"):ArrangeChildrenVert(1)
end

-----------------------------------------------------------------------------------------------
-- SetERBtn
-----------------------------------------------------------------------------------------------
function SAS:SetERBtn() 
	
    self.wndFrame = Apollo.LoadForm(self.Xml, "Frame_Wnd", self.wndSas:FindChild("OptionWnd_Frames"), self) 
    self.wndFrame:SetName("Frame2_Wnd")
    self.wndER_Title = Apollo.LoadForm(self.Xml, "TitleWnd", self.wndSas:FindChild("Frame2_Wnd"), self)  
   	self.wndER_Title:SetText(L["SAS_SWITCHBYER"])
   	self.wndER = Apollo.LoadForm(self.Xml, "OptionForm", self.wndSas:FindChild("Frame2_Wnd"), self)
	self.wndER:FindChild("OptionBtn"):SetData(1)
	self.wndER:SetText(L["SAS_EMERGENCY"])
   	self.wndER_SlideStart = Apollo.LoadForm(self.Xml, "SlideForm", self.wndSas:FindChild("Frame2_Wnd"), self)
	self.wndER_SlideStart:FindChild("SlideBtn"):SetData(2)
	self.wndFrame = Apollo.LoadForm(self.Xml, "Blank_Wnd", self.wndSas:FindChild("Frame2_Wnd"), self) 
    self.wndFrame:SetName("Blank_Wnd")
	self.wndER_BackPrev = Apollo.LoadForm(self.Xml, "LinkForm", self.wndSas:FindChild("Frame2_Wnd"), self)  
    self.wndER_BackPrev:FindChild("LinkBtn"):SetData(3)
    self.wndER_BackPrev:SetText(L["SAS_BACK"])
 	self.wndER_SlidePrev = Apollo.LoadForm(self.Xml, "SlideForm", self.wndSas:FindChild("Frame2_Wnd"), self)
	self.wndER_SlidePrev:FindChild("SlideBtn"):SetData(4)
	self.wndER_SlidePrev:FindChild("SlideBtn"):SetMinMax(self.tSasConfig.er_start, 100)
	self.wndSas:FindChild("Frame2_Wnd"):ArrangeChildrenVert(1)
end

-----------------------------------------------------------------------------------------------
-- OnGetSpeTimer
-----------------------------------------------------------------------------------------------
function SAS:OnGetSpe_Timer()
	
   	local nSpe = AbilityBook.GetCurrentSpec()
   	self.wndSas:FindChild("SpeID"):SetText(tostring(nSpe))
end

-----------------------------------------------------------------------------------------------
-- OnEnteredCombat
-----------------------------------------------------------------------------------------------
function SAS:OnEnteredCombat(unit, bInCombat)
 
   if unit == nil then return end 	   
   if unit:IsThePlayer() then	
		
        if not bInCombat and (self:GetHealthPercent(unit) <= self.tSasConfig.er_start) then
        	self:DoSwitchSPE("ER")
		end

   		self:BlockedLAS(bInCombat)
		self.bInCombat = bInCombat
   end
end


----------------------------------------------------------------------------------------------
-- GetHealthPercent 
-----------------------------------------------------------------------------------------------
function SAS:GetHealthPercent(uTarget)

    if uTarget == nil then return 90 end
	
	local nMaxHealth = uTarget:GetMaxHealth()
	local nHealth = uTarget:GetHealth()
	local nHealthInPerc = math.floor((nHealth / nMaxHealth * 100) + 0.5)

	return nHealthInPerc
end


----------------------------------------------------------------------------------------------
-- BlockedLAS 
-----------------------------------------------------------------------------------------------
function SAS:BlockedLAS(bCombat)
	
	local nSpe = AbilityBook.GetNumUnlockedSpecs()
	
	if nSpe == 1 then
		bCombat = true
		self.wndSas:FindChild("ComboBtn"):Enable(not bCombat)
	end
	
   	self.wndSas:FindChild("SetActionBtn"):Enable(not bCombat)
   	self.wndSas:FindChild("SpeID"):Show(not bCombat)
end


----------------------------------------------------------------------------------------------
-- OnChangeWorld
-----------------------------------------------------------------------------------------------
function SAS:OnChangeWorld()

	if MatchingGame.IsInMatchingGame() == true then 
	  	self:DoSwitchSPE("PvpEnter") 
	elseif MatchingGame.IsInMatchingGame() == false then
		self:DoSwitchSPE("PveEnter")
	end
	
end

----------------------------------------------------------------------------------------------
-- DoSwitchSPE
-----------------------------------------------------------------------------------------------
function SAS:DoSwitchSPE(sWhatSwitch)
 
    local sSwitch = sWhatSwitch
        
    
	if self.bInCombat and sSwitch == "PveEnter" or sSwitch == "PvpEnter" then 
		self:BlockedLAS(false)
	end	
    
	local nSpe = AbilityBook.GetNumUnlockedSpecs()
	local nPrevSpe = AbilityBook.GetCurrentSpec()
	local nStance = GameLib.GetCurrentClassInnateAbilityIndex()
		
	for _=1,nSpe do
		
		if sSwitch == "PvpEnter" and (nPrevSpe ~= _) and self.tSasConfig[_].pvp == true then 
			self.nNewSpe = _
			Apollo.CreateTimer("NewSpe_Timer", 2 , true) 
			break
		end	
		
		if sSwitch == "PveEnter" and (nPrevSpe ~= _) and self.tSasConfig[_].pve == true then   
			self.nNewSpe = _
			Apollo.CreateTimer("NewSpe_Timer", 2 , true)
			break
		end	
		
		if sSwitch == "Stance" and (nPrevSpe ~= _) and self.tSasConfig[_].stance == nStance then   
			self.nNewSpe = _
			AbilityBook.SetCurrentSpec(_)
			break
		end	
				
		if sSwitch == "ER" and (nPrevSpe ~= _) and self.tSasConfig[_].er == true then   
			self.nNewSpe = _
			AbilityBook.SetCurrentSpec(_)
			
			if self.tSasConfig.er_useprev == true then 
				self.nPrevSpe = nPrevSpe
				Apollo.CreateTimer("PrevSpe_Timer", 1 , true)
			end
			break
		end	
	end
end

-----------------------------------------------------------------------------------------------
-- OnButton
-----------------------------------------------------------------------------------------------
function SAS:OnButton( wndHandler, wndControl, eMouseButton )
   
    if self.bHasStance == nil then self.bHasStance = self:ClassHaveStance() end

    if not self.wndSas:FindChild("OptionWnd"):IsShown() then 
		self:CreateOption() 
		self.wndSas:FindChild("OptionWnd_Frames"):ArrangeChildrenVert()
		self:UpdateOption()
	else
		self.wndSas:FindChild("OptionWnd_Frames"):DestroyChildren()
   	end
    self.wndSas:FindChild("OptionWnd"):Show(not self.wndSas:FindChild("OptionWnd"):IsShown())
end

-----------------------------------------------------------------------------------------------
-- ClassHaveStance
-----------------------------------------------------------------------------------------------
function SAS:ClassHaveStance()
	
	local uPlayer = GameLib.GetPlayerUnit()
	local nClassId = uPlayer:GetClassId()
	
	if nClassId <=2 or nClassId == 5 then return true end

	return false	
	
end

-----------------------------------------------------------------------------------------------
-- CreateOption
-----------------------------------------------------------------------------------------------
function SAS:CreateOption()

	self:SetZoneBtn()
		
	if self.bHasStance == false then 
		self:SetERBtn()
		return
	end
	
	self:SetStanceBtn()
	
	if self.bHasStance then self:SetOptionSize() end

end


-----------------------------------------------------------------------------------------------
-- SetOptionSize
-----------------------------------------------------------------------------------------------
function SAS:SetOptionSize()

    if self.nLastStance == self.nStanceCount then return end

	local nBtnOptionsSize = 15 * self.nStanceCount
	local l,t,r,b = self.wndSas:FindChild("OptionWnd"):GetAnchorOffsets()
	
	self.wndSas:FindChild("OptionWnd"):SetAnchorOffsets(l, (t + nBtnOptionsSize), r, b)
	
	self.nLastStance = self.nStanceCount
end


-----------------------------------------------------------------------------------------------
-- UpdateOption
-----------------------------------------------------------------------------------------------
function SAS:UpdateOption()
	
	local nSpe = AbilityBook.GetCurrentSpec()
    
		
	self.arZones[1].wndZone:FindChild("OptionBtn"):SetCheck(self.tSasConfig[nSpe].pvp)
	self.arZones[2].wndZone:FindChild("OptionBtn"):SetCheck(self.tSasConfig[nSpe].pve)
		
	
	if self.bHasStance == false then 
		self.wndER:FindChild("OptionBtn"):SetCheck(self.tSasConfig[nSpe].er)
		self.wndER_SlideStart:FindChild("SlideBtn"):SetValue(self.tSasConfig.er_start)
		self.wndER_SlideStart:SetText(L["SAS_HEALTHBELOW"] .. tostring(self.tSasConfig.er_start) .. "%")
		self.wndER_BackPrev:FindChild("LinkBtn"):SetCheck(self.tSasConfig.er_useprev)
		self.wndER_SlidePrev:FindChild("SlideBtn"):SetValue(self.tSasConfig.er_prev)
		self.wndER_SlidePrev:SetText(L["SAS_HEALTHABOVE"] .. tostring(self.tSasConfig.er_prev) .. "%")

		return
	end
	
	for _=1,self.nStanceCount do
		if _ == self.tSasConfig[nSpe].stance then 
			self.arStances[_].wndStance:FindChild("OptionBtn"):SetCheck(true)
		else
		 	self.arStances[_].wndStance:FindChild("OptionBtn"):SetCheck(false)
		end
    end

   	self.wndLink_Stance:FindChild("LinkBtn"):SetCheck(self.tSasConfig.link)
end



---------------------------------------------------------------------------------------------------
-- InComingSpe
---------------------------------------------------------------------------------------------------
function SAS:InComingSpe()
   	   
		local nCurSpe = AbilityBook.GetCurrentSpec()
		local nSpe = AbilityBook.GetNumUnlockedSpecs()
		self.nNewSpe = nCurSpe + 1
		if nCurSpe == nSpe then self.nNewSpe = 1 end
end



---------------------------------------------------------------------------------------------------
-- OnSetAction
---------------------------------------------------------------------------------------------------
function SAS:OnSetAction( wndHandler, wndControl, eMouseButton )
   	   
		self:InComingSpe()
		AbilityBook.NextSpec()
end

---------------------------------------------------------------------------------------------------
-- OnOptionClick 		[ For all EVENT 'CHECKBOX BUTTON' Option ]
---------------------------------------------------------------------------------------------------
function SAS:OnOptionClick( wndHandler, wndControl, eMouseButton )
   
	local cParent = wndControl:GetParent():GetParent():GetName() 
     
	local nSpe = AbilityBook.GetCurrentSpec()
	
	 if cParent == "Frame2_Wnd" then 
			
	        if self.bHasStance then
				if wndControl:GetData() <= self.nStanceCount then
					
					self.tSasConfig[nSpe].stance = 0
			    end
				if wndControl:GetData() == self.nStanceCount + 1 then
					
					self.tSasConfig.link = false
				end
			end	
			
		
			if not self.bHasStance then
				if wndControl:GetData() == 1 then
				
					self.tSasConfig[nSpe].er = false
			    end
				if wndControl:GetData() == 3 then
					
					self.tSasConfig.er_useprev = false
				end
			end 
		
	
	 end
			
	 
	 if cParent == "Frame1_Wnd" then
			
	 		self.tSasConfig[nSpe].pve = false
	 		self.tSasConfig[nSpe].pvp = false
	 end	

		
   	
    if wndControl:IsChecked() then 
      
        if cParent == "Frame2_Wnd" then
			
			if self.bHasStance then
				
				if wndControl:GetData() <= self.nStanceCount then
					
    				self.tSasConfig[nSpe].stance = wndControl:GetData()
	
					for _=1,self.nStanceCount do 
						if _ ~= wndControl:GetData() then self.arStances[_].wndStance:FindChild("OptionBtn"):SetCheck(false) end
					end
				
					for _=1,#self.tSasConfig do 
						if _~= nSpe and (self.tSasConfig[_].stance == wndControl:GetData()) then self.tSasConfig[_].stance = 0 end
        			end
                end

				if wndControl:GetData() == self.nStanceCount + 1 then
					
    				self.tSasConfig.link = true
                end

			end
            
			if not self.bHasStance then
			
				
				if wndControl:GetData() == 1 then

				
					self.tSasConfig[nSpe].er = wndControl:IsChecked()
					
					for _=1,#self.tSasConfig do 
						if _~= nSpe then self.tSasConfig[_].er = false end
       				end
    			end

				if wndControl:GetData() == 3 then
					self.tSasConfig.er_useprev = true
                end



	        end
			
		end
	   
        if cParent == "Frame1_Wnd" then
			
    		if wndControl:GetData() == 1 then 
					self.tSasConfig[nSpe].pvp = wndControl:IsChecked()
					self.tSasConfig[nSpe].pve = false
			end		
		
        	if wndControl:GetData() == 2 then
					self.tSasConfig[nSpe].pve = wndControl:IsChecked()
					self.tSasConfig[nSpe].pvp = false
			end		
		
			for _=1,#self.tSasConfig do 
				if _~= nSpe and wndControl:GetData() == 1 then self.tSasConfig[_].pvp = false end
       			if _~= nSpe and wndControl:GetData() == 2 then self.tSasConfig[_].pve = false end
	    	end
		end	
	
	end
	
	self:UpdateOption()
end


---------------------------------------------------------------------------------------------------
-- OnOptionSlide 		[ For all EVENT 'SLIDEBAR' Option ]
---------------------------------------------------------------------------------------------------
function SAS:OnOptionSlide( wndHandler, wndControl, fNewValue, fOldValue )
	
    local cParent = wndControl:GetParent():GetParent():GetName() 
	local nNewValue = math.floor(fNewValue + 0.5)
     

	if cParent == "Frame2_Wnd" then 
	
				if wndControl:GetData() == 2 then
					self.tSasConfig.er_start = nNewValue
					self.wndER_SlideStart:SetText(L["SAS_HEALTHBELOW"] .. tostring(nNewValue) .. "%")
					local nMinBack = nNewValue
					self.wndER_SlidePrev:FindChild("SlideBtn"):SetMinMax(nMinBack, 100)
					
					if self.tSasConfig.er_prev < nMinBack then
						self.wndER_SlidePrev:SetText(L["SAS_HEALTHABOVE"] .. tostring(nMinBack) .. "%")
						self.tSasConfig.er_prev = nNewValue
					end
                end

				if wndControl:GetData() == 4 then
					
    				self.tSasConfig.er_prev = nNewValue
					self.wndER_SlidePrev:SetText(L["SAS_HEALTHABOVE"] .. tostring(nNewValue) .. "%")
                end

    end
end



-----------------------------------------------------------------------------------------------
-- OnSpecChanged
-----------------------------------------------------------------------------------------------
function SAS:OnSpecChanged(newSpecIndex, specError)
		
       
		if (self.nNewSpe == newSpecIndex or self.nPrevSpe == newSpecIndex) and specError == AbilityBook.CodeEnumSpecError.Ok then	
		
			if self.bNewSpe_Timer then 
				Apollo.StopTimer("NewSpe_Timer")
			
				self.bNewSpe_Timer = nil
			elseif self.bPrevSpe_Timer then
				Apollo.StopTimer("PrevSpe_Timer")
				
				self.nPrevSpe = 0
				self.bPrevSpe_Timer = nil
			end
			
			self.nNewSpe = 0
			
        end


		self.wndSas:FindChild("SpeID"):SetText(tostring(newSpecIndex))
		
		if self.tSasConfig.link == true and self.tSasConfig[newSpecIndex].stance ~= 0 and self.nStance ~= self.tSasConfig[newSpecIndex].stance then   
			
				GameLib.SetCurrentClassInnateAbilityIndex(self.tSasConfig[newSpecIndex].stance)
		end	

		if self.wndSas:FindChild("OptionWnd"):IsShown() then self:UpdateOption() end

end


-----------------------------------------------------------------------------------------------
-- OnNewSpe_Timer
-----------------------------------------------------------------------------------------------
function SAS:OnNewSpe_Timer()
	
	self.bNewSpe_Timer = true
 	AbilityBook.SetCurrentSpec(self.nNewSpe)
end


-----------------------------------------------------------------------------------------------
-- OnPrevSpe_Timer (FOR 'ER' PREVIOUS)
-----------------------------------------------------------------------------------------------
function SAS:OnPrevSpe_Timer()
	
	self.bPrevSpe_Timer = true
 
    if not self.bInCombat and (self:GetHealthPercent(GameLib.GetPlayerUnit()) >= self.tSasConfig.er_prev) then
        	AbilityBook.SetCurrentSpec(self.nPrevSpe)
	end

end





-----------------------------------------------------------------------------------------------
-- OnSave
-----------------------------------------------------------------------------------------------
function SAS:OnSave(eLevel)
	
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return nil end
   
	local l,t,r,b = self.wndSas:GetAnchorOffsets()
	self.tSasConfig.anchor = { l = l, t = t, r = r, b = b }
		
	local tTable = {}
	tTable = self.tSasConfig
						 
	return tTable
end


-----------------------------------------------------------------------------------------------
-- OnRestore
-----------------------------------------------------------------------------------------------
function SAS:OnRestore(eLevel, tData)
			
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return nil end
	   				
	local tSasConfigTmp = self:TableMerge(self.tSasConfig, tData)
	self.tSasConfig = tSasConfigTmp
	
	local tPos = self.tSasConfig.anchor
	
	if tPos ~= nil then 
		self.wndSas:SetAnchorOffsets(tPos.l, tPos.t, tPos.r, tPos.b)
	end 
end


-----------------------------------------------------------------------------------------------
-- TableMerge 
-----------------------------------------------------------------------------------------------
function SAS:TableMerge(t1, t2)
    for k,v in pairs(t2) do
    	if type(v) == "table" then
    		if type(t1[k] or false) == "table" then
    			self:TableMerge(t1[k] or {}, t2[k] or {})
    		else
    			t1[k] = v
    		end
    	else
    		t1[k] = v
    	end
    end
    return t1
end


---------------------------------------------------------------------------------------------------
-- OnToolTip
---------------------------------------------------------------------------------------------------
function SAS:OnToolTip( wndHandler, wndControl, x, y )
	self.wndSas:FindChild("SetActionBtn"):SetTooltip("\'SAS\' v " .. SAS_VER)	
end



-----------------------------------------------------------------------------------------------
-- SAS Instance
-----------------------------------------------------------------------------------------------
local SASInst = SAS:new()
SASInst:Init()
